using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Rendering;
using Microsoft.EntityFrameworkCore;
using MapsDb;
using MapsDb.Interfaces;
using MapsDb.DataService;
using MapsModels.ViewModels;
using MapsModels.DsModels;
using System;

namespace Maps.Controllers
{
    public class RoadToCategoryController : Controller
    {
        private readonly IRoadDs _roadDs;
        private readonly IRegionDs _regionDs;
        private readonly IRoadToCategoryDs _roadToCategoryDs;
        private readonly IRoadCategoryDs _roadCategoryDs;
        public RoadToCategoryController(
            IRoadDs RoadDs, 
            IRegionDs RegionDs,
            IRoadCategoryDs RoadCategoryDs,
            IRoadToCategoryDs RoadToCategoryDs
            )
        {
            _roadDs = RoadDs;     
            _regionDs = RegionDs;       
            _roadCategoryDs = RoadCategoryDs;
            _roadToCategoryDs = RoadToCategoryDs;
        }

        // GET: BusStop
        [HttpGet]
        public async Task<IActionResult> Index([FromQuery] PaginationDsM data)
        {

            try
            {
                var Data = await _roadToCategoryDs.GetIndexListAsync(data);

                RoadToCategoryListVm vm = new RoadToCategoryListVm
                {
                    RoadToCategoryEditDsM = Data.ToList()
                };

                return Json(vm);
            }
            catch (NullReferenceException)
            {
                Response.StatusCode = 400;
                return Json("There is no field with name " + data.sort);
            }
            catch (Exception)
            {
                return NotFound();
            }
        }

        [HttpGet]
        public async Task<IActionResult> Directory(){
            var Road = await _roadDs.GetSelectListAsync();
            var Region = await _regionDs.GetSelectListAsync();
            var RoadCategory = await _roadCategoryDs.GetSelectListAsync();
            CatalogListVm vm = new CatalogListVm
            {
                RoadSelectListDsM = Road.ToList(),
                RegionSelectListDsM = Region.ToList(),
                RoadCategorySelectListDsM = RoadCategory.ToList()
            };
            return Json(vm);
        }


        [HttpPost]
        public async Task<IActionResult> Create([FromBody] RoadToCategoryEditDsM data)
        {
            var result = await _roadToCategoryDs.CreateAsync(data);
            return Json(result);
        }

        [HttpPost]
        public async Task<IActionResult> Update(int id, [FromBody] RoadToCategoryEditDsM data){
                await _roadToCategoryDs.UpdateAsync(data,id);
                return Json(String.Empty);
        }

     
        [HttpDelete]
        public async Task<IActionResult> Delete(int id)
        {   
            try
            {
                 int data = await _roadToCategoryDs.DeleteAsync(id);
                 return Json(data);
            }
            catch (ArgumentNullException )
            {
                return NotFound();
            }
        }
    }
}
