﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using Microsoft.EntityFrameworkCore;
using MapsDb;
using MapsDb.Interfaces;
using MapsDb.DataService;
using MapsModels;
using AutoMapper;
using MapsDb.Models;
using MapsModels.DsModels;

namespace Maps
{
    public class Startup
    {
        public Startup(IHostingEnvironment env)
        {
            var builder = new ConfigurationBuilder()
                .SetBasePath(env.ContentRootPath)
                .AddJsonFile("appsettings.json", optional: true, reloadOnChange: true)
                .AddJsonFile($"appsettings.{env.EnvironmentName}.json", optional: true)
                .AddEnvironmentVariables();

            if (env.IsDevelopment())
            {
                // This will push telemetry data through Application Insights pipeline faster, allowing you to view results immediately.
                builder.AddApplicationInsightsSettings(developerMode: true);
            }
            Configuration = builder.Build();
        }

        public IConfigurationRoot Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
             services.AddCors(o => o.AddPolicy("MyPolicy", builder => {
                 builder.AllowAnyOrigin()
                        .AllowAnyMethod()
                        .AllowAnyHeader();
             }));
            services.AddScoped<PostgresDbContext>();

            Mapper.Initialize(cnf => {
                cnf.CreateMap<BusStop, BusStopEditDsM>();
                cnf.CreateMap<BusStopEditDsM, BusStop>();
                cnf.CreateMap<RoadService, RoadServiceEditDsM>();
                cnf.CreateMap<RoadServiceEditDsM, RoadService>();
                cnf.CreateMap<SettlementAddressLink, SettlementAddressLinkEditDsM>();
                cnf.CreateMap<SettlementAddressLinkEditDsM, SettlementAddressLink>();
                cnf.CreateMap<RoadToCategory, RoadToCategoryEditDsM>();
                cnf.CreateMap<RoadToCategoryEditDsM, RoadToCategory>();
                cnf.CreateMap<AuthorityEditDsM, Authority>();
                cnf.CreateMap<Authority, AuthorityEditDsM>();
                cnf.CreateMap<ContractorEditDsM, Contractor>();
                cnf.CreateMap<Contractor, ContractorEditDsM>();
            });

            services.AddScoped<IBusStopDs, BusStopDs>();
            services.AddScoped<IRoadDs, RoadDs>();
            services.AddScoped<IRegionDs, RegionDs>();
            services.AddScoped<IStateCommonDs, StateCommonDs>();
            services.AddScoped<ISurfaceTypeDs, SurfaceTypeDs>();
            services.AddScoped<IServiceObjectDs, ServiceObjectDs>();
            services.AddScoped<IServiceObjectTypeDs, ServiceObjectTypeDs>();
            services.AddScoped<ISettlementDs, SettlementDs>();
            services.AddScoped<IDepartmentAffiliationDs, DepartmentAffiliationDs>();
            services.AddScoped<ISurfaceTreatmentDs, SurfaceTreatmentDs>();
            services.AddScoped<IRoadDirectionDs, RoadDirectionDs>();
            services.AddScoped<IRoadSurfaceDs, RoadSurfaceDs>();
            services.AddScoped<IRoadWidthDs, RoadWidthDs>();
            services.AddScoped<IFlowIntensityDs, FlowIntensityDs>();
            services.AddScoped<ICrossSectionDs, CrossSectionDs>();
            services.AddScoped<IRoadTypeDs, RoadTypeDs>();
            services.AddScoped<IOrganizationDs, OrganizationDs>();
            services.AddScoped<IRoadServiceDs, RoadServiceDs>();
            services.AddScoped<ISettlementLocationDs, SettlementLocationDs>();
            services.AddScoped<ISettlementAddressLinkDs, SettlementAddressLinkDs>();
            services.AddScoped<IRoadToCategoryDs, RoadToCategoryDs>();
            services.AddScoped<IRoadCategoryDs, RoadCategoryDs>();
            services.AddScoped<IAuthorityDs, AuthorityDs>();
            services.AddScoped<IContractorDs, ContractorDs>();
            // Add framework services.
            services.AddApplicationInsightsTelemetry(Configuration);

            services.AddMvc();

        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IHostingEnvironment env, ILoggerFactory loggerFactory)
        {

          app.UseCors("MyPolicy");
            app.UseDeveloperExceptionPage();
            app.UseBrowserLink();
 
            loggerFactory.AddConsole(Configuration.GetSection("Logging"));
            loggerFactory.AddDebug();

            app.UseApplicationInsightsRequestTelemetry();
            app.UseApplicationInsightsExceptionTelemetry();

            app.UseStaticFiles();

            app.UseMvc(routes =>
            {
                routes.MapRoute(
                    name: "default",
                    template: "{controller=Home}/{action=Index}/{id?}");
            });
        }
    }
}
