using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using AutoMapper;
using MapsDb.Interfaces;
using MapsDb.Models;
using MapsModels.DsModels;
using Microsoft.EntityFrameworkCore;

namespace MapsDb.DataService
{
    public class AuthorityDs : IAuthorityDs
    {
        private PostgresDbContext _context;
        public AuthorityDs(){
            _context = new PostgresDbContext();
        }
        public Task<IList<AuthorityEditDsM>> GetIndexListAsync(PaginationDsM pagination){
            return Task.Factory.StartNew(()=> { return GetAllAuthority(pagination); });
        }
        private IList<AuthorityEditDsM> GetAllAuthority(PaginationDsM pagination)
        {

            var filter =  pagination.filter;

            IQueryable<Authority> data = _context.Authority
            .Include(d=>d.Road)
            .Include(d=>d.CrossSection);

            data = Filtering(pagination.filter, data);

            IQueryable<AuthorityEditDsM> result = data
            .Select(Authority => Mapper.Map<AuthorityEditDsM>(Authority))
            .Skip(pagination.from)
            .Take(pagination.perPage);

            switch (pagination.orderType())
            {
                case "ASC":
                    return result.OrderBy(i => i.GetType().GetProperty(pagination.sort).GetValue(i, null)).ToList();
          
                case "DESC":
                    return result.OrderByDescending(i => i.GetType().GetProperty(pagination.sort).GetValue(i, null)).ToList();
            
                default:
                    return result.OrderByDescending(i => i.Id).ToList();
            }     
            
        }

        public Task<Authority> CreateAsync(AuthorityEditDsM data){
              return Task.Factory.StartNew(()=> { return Create(data); });
        }
        private Authority Create(AuthorityEditDsM data)
        {   
            
            Authority Model = InsertModel(data);
            _context.Authority.Add(Model);
            _context.SaveChanges();
            return Model;
        }
        public Task<Authority> UpdateAsync(AuthorityEditDsM data, int id){
            return Task.Factory.StartNew(()=> { return Update(data, id); });
        }
        private Authority Update(AuthorityEditDsM data, int id)
        {   
            Authority Model = InsertModel(data);
            Model.Id = id;
            _context.Authority.Update(Model);
            _context.SaveChanges();
            return Model;
        }
        public Authority InsertModel(AuthorityEditDsM data){
            Authority Model = Mapper.Map<Authority>(data);
            return Model;
        }
        public async Task<int> DeleteAsync(int Id)
        {
            var authority = await _context.Authority.SingleOrDefaultAsync(x => x.Id == Id);
            _context.Authority.Remove(authority);
            return await _context.SaveChangesAsync();
        }
        public IQueryable<Authority> Filtering(string filters, IQueryable<Authority> data){
            if(filters != null){
               string[] FilterWords = filters.Split(';').Where(x => !string.IsNullOrEmpty(x)).ToArray();
                foreach(string word in FilterWords){
                    string[] filter = Regex.Split(word.Substring(1), "(.*)_(.*)");
                    if(filter.Length < 3 ){
                        continue;
                    }
                    string field = char.ToUpper(filter[1][0]) + filter[1].Substring(1);
                    string param = filter[2].ToLower();
                    if(word.StartsWith("$")){
                        data = FilterByEndWith(field, param, data);
                    }
                    else if(word.StartsWith("^")){
                        data = FilterByStartWith(field, param, data);
                    }
                    else if(word.StartsWith("*")){
                        data = FilterByComtains(field, param, data);
                    }
                    else if(word.StartsWith("!")){
                        data = FilterByNotEquals(field, param, data);
                    }
                    else if(word.StartsWith("=")){
                        data = FilterByEquals(field, param, data);
                    }
                }
            
            }
            return data;
        }
        public IQueryable<Authority> FilterByComtains(string field, string param, IQueryable<Authority> data){
            switch(field){
                case "RoadId":
                    return data.Where(i => i.Road.Name != null && i.Road.Name.ToLower().Contains(param) );
                case "CrossSectionId":
                    return data.Where(i => i.CrossSection.Id.ToString().ToLower().Contains(param) );
                default:
                    return data.Where(i => Convert.ToString(i.GetType().GetProperty(field).GetValue(i, null)).ToLower().Contains(param) );
            }
        }
        public IQueryable<Authority> FilterByEquals(string field, string param, IQueryable<Authority> data){
            switch(field){
                case "RoadId":
                    return data.Where(i => i.Road.Id.ToString().ToLower() == param );
                case "CrossSectionId":
                    return data.Where(i => i.CrossSection.Id.ToString().ToLower() == param );
                default:
                return data.Where(i => Convert.ToString(i.GetType().GetProperty(field).GetValue(i, null)).ToLower() == param );
            }

        }
        public IQueryable<Authority> FilterByNotEquals(string field, string param, IQueryable<Authority> data){
           switch(field){
                case "RoadId":
                    return data.Where(i => i.Road.Name != null && i.Road.Name.ToLower() != param );
                case "CrossSectionId":
                    return data.Where(i => i.CrossSection.Id.ToString().ToLower() != param );
                default:
                return data.Where(i => Convert.ToString(i.GetType().GetProperty(field).GetValue(i, null)).ToLower() != param );
            }
        }
        public IQueryable<Authority> FilterByStartWith(string field, string param, IQueryable<Authority> data){
            switch(field){
                case "RoadId":
                    return data.Where(i => i.Road.Name != null && i.Road.Name.ToLower().StartsWith(param) );
                case "CrossSectionId":
                    return data.Where(i => i.CrossSection.Id.ToString().ToLower().StartsWith(param) );
                default:
                return data.Where(i => Convert.ToString(i.GetType().GetProperty(field).GetValue(i, null)).ToLower().StartsWith(param) );
            }
        }
        public IQueryable<Authority> FilterByEndWith(string field, string param, IQueryable<Authority> data){
          switch(field){
                case "RoadId":
                    return data.Where(i => i.Road.Name != null && i.Road.Name.ToLower().EndsWith(param) );
                case "CrossSectionId":
                    return data.Where(i => i.CrossSection.Id.ToString().ToLower().EndsWith(param) );
                default:
                return data.Where(i => Convert.ToString(i.GetType().GetProperty(field).GetValue(i, null)).ToLower().EndsWith(param) );
            }
        }
    }
}