using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using AutoMapper;
using MapsDb.Interfaces;
using MapsDb.Models;
using MapsModels.DsModels;
using Microsoft.EntityFrameworkCore;

namespace MapsDb.DataService
{
    public class ContractorDs : IContractorDs
    {
        private PostgresDbContext _context;
        public ContractorDs(){
            _context = new PostgresDbContext();
        }
        public Task<IList<ContractorEditDsM>> GetIndexListAsync(PaginationDsM pagination){
            return Task.Factory.StartNew(()=> { return GetAllContractor(pagination); });
        }
        private IList<ContractorEditDsM> GetAllContractor(PaginationDsM pagination)
        {

            var filter =  pagination.filter;

            IQueryable<Contractor> data = _context.Contractor
            .Include(d=>d.Road)
            .Include(d=>d.CrossSection);

            data = Filtering(pagination.filter, data);

            IQueryable<ContractorEditDsM> result = data
            .Select(Contractor => Mapper.Map<ContractorEditDsM>(Contractor))
            .Skip(pagination.from)
            .Take(pagination.perPage);

            switch (pagination.orderType())
            {
                case "ASC":
                    return result.OrderBy(i => i.GetType().GetProperty(pagination.sort).GetValue(i, null)).ToList();
          
                case "DESC":
                    return result.OrderByDescending(i => i.GetType().GetProperty(pagination.sort).GetValue(i, null)).ToList();
            
                default:
                    return result.OrderByDescending(i => i.Id).ToList();
            }     
            
        }

        public Task<Contractor> CreateAsync(ContractorEditDsM data){
              return Task.Factory.StartNew(()=> { return Create(data); });
        }
        private Contractor Create(ContractorEditDsM data)
        {   
            
            Contractor Model = InsertModel(data);
            _context.Contractor.Add(Model);
            _context.SaveChanges();
            return Model;
        }
        public Task<Contractor> UpdateAsync(ContractorEditDsM data, int id){
            return Task.Factory.StartNew(()=> { return Update(data, id); });
        }
        private Contractor Update(ContractorEditDsM data, int id)
        {   
            Contractor Model = InsertModel(data);
            Model.Id = id;
            _context.Contractor.Update(Model);
            _context.SaveChanges();
            return Model;
        }
        public Contractor InsertModel(ContractorEditDsM data){
            Contractor Model = Mapper.Map<Contractor>(data);
            return Model;
        }
        public async Task<int> DeleteAsync(int Id)
        {
            var contractor = await _context.Contractor.SingleOrDefaultAsync(x => x.Id == Id);
            _context.Contractor.Remove(contractor);
            return await _context.SaveChangesAsync();
        }
        public IQueryable<Contractor> Filtering(string filters, IQueryable<Contractor> data){
            if(filters != null){
               string[] FilterWords = filters.Split(';').Where(x => !string.IsNullOrEmpty(x)).ToArray();
                foreach(string word in FilterWords){
                    string[] filter = Regex.Split(word.Substring(1), "(.*)_(.*)");
                    if(filter.Length < 3 ){
                        continue;
                    }
                    string field = char.ToUpper(filter[1][0]) + filter[1].Substring(1);
                    string param = filter[2].ToLower();
                    if(word.StartsWith("$")){
                        data = FilterByEndWith(field, param, data);
                    }
                    else if(word.StartsWith("^")){
                        data = FilterByStartWith(field, param, data);
                    }
                    else if(word.StartsWith("*")){
                        data = FilterByComtains(field, param, data);
                    }
                    else if(word.StartsWith("!")){
                        data = FilterByNotEquals(field, param, data);
                    }
                    else if(word.StartsWith("=")){
                        data = FilterByEquals(field, param, data);
                    }
                }
            
            }
            return data;
        }
        public IQueryable<Contractor> FilterByComtains(string field, string param, IQueryable<Contractor> data){
            switch(field){
                case "RoadId":
                    return data.Where(i => i.Road.Name != null && i.Road.Name.ToLower().Contains(param) );
                case "CrossSectionId":
                    return data.Where(i => i.CrossSection.Id.ToString().ToLower().Contains(param) );
                default:
                    return data.Where(i => Convert.ToString(i.GetType().GetProperty(field).GetValue(i, null)).ToLower().Contains(param) );
            }
        }
        public IQueryable<Contractor> FilterByEquals(string field, string param, IQueryable<Contractor> data){
            switch(field){
                case "RoadId":
                    return data.Where(i => i.Road.Id.ToString().ToLower() == param );
                case "CrossSectionId":
                    return data.Where(i => i.CrossSection.Id.ToString().ToLower() == param );
                default:
                return data.Where(i => Convert.ToString(i.GetType().GetProperty(field).GetValue(i, null)).ToLower() == param );
            }

        }
        public IQueryable<Contractor> FilterByNotEquals(string field, string param, IQueryable<Contractor> data){
           switch(field){
                case "RoadId":
                    return data.Where(i => i.Road.Name != null && i.Road.Name.ToLower() != param );
                case "CrossSectionId":
                    return data.Where(i => i.CrossSection.Id.ToString().ToLower() != param );
                default:
                return data.Where(i => Convert.ToString(i.GetType().GetProperty(field).GetValue(i, null)).ToLower() != param );
            }
        }
        public IQueryable<Contractor> FilterByStartWith(string field, string param, IQueryable<Contractor> data){
            switch(field){
                case "RoadId":
                    return data.Where(i => i.Road.Name != null && i.Road.Name.ToLower().StartsWith(param) );
                case "CrossSectionId":
                    return data.Where(i => i.CrossSection.Id.ToString().ToLower().StartsWith(param) );
                default:
                return data.Where(i => Convert.ToString(i.GetType().GetProperty(field).GetValue(i, null)).ToLower().StartsWith(param) );
            }
        }
        public IQueryable<Contractor> FilterByEndWith(string field, string param, IQueryable<Contractor> data){
          switch(field){
                case "RoadId":
                    return data.Where(i => i.Road.Name != null && i.Road.Name.ToLower().EndsWith(param) );
                case "CrossSectionId":
                    return data.Where(i => i.CrossSection.Id.ToString().ToLower().EndsWith(param) );
                default:
                return data.Where(i => Convert.ToString(i.GetType().GetProperty(field).GetValue(i, null)).ToLower().EndsWith(param) );
            }
        }
    }
}